# -*- coding:utf8 -*-

import csv
import StringIO

from xml.etree import ElementTree
from collections import defaultdict

def convert(source_file, destination_file, summarize):
    """
    steuert die einzelnenn funktionen an und bringt den ausgabe string in
    das richtinge encoding
    """
    places = parse_xml(source_file)


    #with open(destination_file, 'wb') as file_obj:
    file_obj = StringIO.StringIO()
    csv_writer = csv.writer(file_obj, delimiter=';', escapechar="\"")

    if summarize:
        write_summarized(places, csv_writer)

    else:
        write(places, csv_writer)



    with open(destination_file, 'wb') as file_obj2:
        file_obj2.write(file_obj.getvalue().decode('utf8').encode('latin1'))

def write_summarized(places, csv_writer):
    """
    schreibt die daten leicht zusammengefast so das nur ein eintrag pro reihe
    in der Zieldatei landet
    """
    csv_writer.writerow(('Level', 'Section', 'Reihe', 'Plätze'))

    for level in places:
        for section in places[level]:
            for row in places[level][section]:
                place_amount = len(places[level][section][row])

                csv_writer.writerow((level, section, row, place_amount))


def write(places, csv_writer):
    """
    schreibt jeden einzelnen platz in die Zieldatei
    """
    csv_writer.writerow(('Level', 'Section', 'Reihe', 'Platz'))

    for level in places:
        for section in places[level]:
            for row in places[level][section]:
                for place in places[level][section][row]:
                    csv_writer.writerow((level, section, row, place))


def parse_xml(file_name):
    """
    parsed die xml datei und gibt die pläze als dict zurück
    """
    tree = ElementTree.parse(file_name)

    level_names = extract_level_names(tree)
    section_names = extract_section_names(tree)

    places = defaultdict(lambda: defaultdict(lambda: defaultdict(list)))

    for s in tree.findall('physical/pgs/np/sg/s'):
        level =  s.get('le', '').encode('utf8')
        section = s.get('se', '').encode('utf8')
        row_number = s.get('rn', '')
        place_number = int(s.get('sn', ''))

        if level:
            level = level_names[level]

        if section:
            section = section_names[section]

        places[level][section][row_number].append(place_number)

    return places


def extract_level_names(tree):
    """
    liest die level namen aus der xml datei aus
    """

    level_names = {}

    for lelv_e in tree.getiterator('levl'):
        for le in lelv_e:
            if le.tag == 'le':
                for text in le:
                    if text.attrib['loc'] == 'de_DE':
                        level_names[le.attrib['c'].encode('utf8')] = text.attrib['t'].encode('utf8')

    return level_names


def extract_section_names(tree):
    """
    list die sections namen aus der xml datei aus
    """

    section_names = {}

    for secl_e in tree.getiterator('secl'):
        for se in secl_e:
            if se.tag == 'se':
                for text in se:
                    if text.attrib['loc'] == 'de_DE':
                        section_names[se.attrib['c'].encode('utf8')] = text.attrib['t'].encode('utf8')


    return section_names
